function analysis_withSavedData_Mux(process_data,n_Approach,n_Cov,...
    str_visit,n_typeFeat,iter)
%
% analysis_withSavedData_Mux.m
% % process_data: true,false 
% %               false:Show results, true: processing data
% % n_Approach:   1,2 
% %               1:Two group comparison, 2: Suvival analysis
% % n_Cov:        1,2 
% %               1:Age, 2: Age+gender+education
% % str_visit:    'bl','m06','m12','m18','m24','m36'
% %               
% % n_typeFeat:   1,2 
% %               1:MRI, 2: MRI+NM
% % iter:         iteration of k-fold
% % analysis_withSavedData_Mux(process_data,n_Approach,n_Cov,str_visit,n_typeFeat,iter)
% 
%
% Author: Carlos Platero
% Date: 2019/07/21
% 
% General inquiries: carlos.platero@upm.es
%

clf;
clc;
typeModels={'LME','extCox_LME'};
varModels=cellstr(typeModels{n_Approach});
typeCovariables={'Ag','AgSxEd'};
varCovar=cellstr(typeCovariables{n_Cov});
typeVisits={'bl','m06','m12','m18','m24','m36'};
typeFeatures={'ROI','ROICAM'};
typeFeat=cellstr(typeFeatures{n_typeFeat});


%process_data
close all;
addpath('./kfolds_aux');

%% files 
dir_mat=char(strcat('./Results/',varModels,'/'));
kfile=char(strcat('kf_',varModels,varCovar,'_',str_visit,'_',typeFeat,...
    '_',num2str(iter)));
betterFile=char(strcat('better_',varModels,varCovar,'_',str_visit,...
    '_',typeFeat,'_',num2str(iter)));


kfile=      strcat(dir_mat,kfile);
betterFile= strcat(dir_mat,betterFile);

%% data
%dimension of features willing to check
if(strcmp(typeFeat,'ROI'))
    dim_feat=2:4;
else
    dim_feat=5:7;
end

%number of best combinations used for each dimension and each fold:
ngroups=3;

%inner_kfolds
inner_kfolds=10;

%external_kfolds
external_kfolds=10;

%repeat_kfolds
repeat_kf=iter/external_kfolds;


% The k-first features most appear
ngroups_forDim=10;


%% Processing
if(process_data)

    load(kfile,'sel','stat_feat','feature_names');

    sel=sel(:,:,1:external_kfolds*repeat_kf);
    stat_feat=stat_feat(:,:,:,1:external_kfolds*repeat_kf);

    [sel_ord,stat_ord]=sortScores(ngroups,inner_kfolds,external_kfolds*repeat_kf,...
        dim_feat,sel,squeeze(stat_feat(:,:,1,:)));
    [~,stat_bsl]=sortScores(ngroups,inner_kfolds,external_kfolds*repeat_kf,...
        dim_feat,sel,squeeze(stat_feat(:,:,2,:)));

    [betterCombs,betterScores,betterBslScores,ci_scores]=chooseBestCombinationEachDimension(...
        sel_ord,stat_ord,stat_bsl,dim_feat,ngroups_forDim);
     save(betterFile,'betterCombs','betterScores','betterBslScores','feature_names','ci_scores');

else
    load(betterFile,'betterCombs','betterScores','betterBslScores','feature_names','ci_scores');
    clc;
    kfirts_list=5;
    fprintf('Results for %s, %s, %s:\n', string(varModels),string(varCovar),...
        string(str_visit));
    printf_long_results(betterCombs,betterBslScores,feature_names,kfirts_list,squeeze(ci_scores(:,:,2)));
    fprintf('Press any key to continue\n');
    pause;
     
end



rmpath('./kfolds_aux');

end

function [sel_ord,stat_ord]=sortScores(ngroups,inner_kfolds,external_kfolds,...
    dim_feat,sel,stat_feat)


sel_ord=zeros(ngroups*inner_kfolds*external_kfolds,max(dim_feat),length(dim_feat));
stat_ord=zeros(ngroups*inner_kfolds*external_kfolds,6,length(dim_feat));
for i=1:length(dim_feat)
    sel_dim=sel((ngroups*inner_kfolds*(i-1))+1:ngroups*inner_kfolds*i,:,:);
    sel_ord(:,:,i)=reshape(permute(sel_dim,[1 3 2]),[ngroups*inner_kfolds*external_kfolds,max(dim_feat)]);
    stat_dim=stat_feat((ngroups*inner_kfolds*(i-1))+1:ngroups*inner_kfolds*i,:,:);
    stat_ord(:,:,i)=reshape(permute(stat_dim,[1 3 2]),[ngroups*inner_kfolds*external_kfolds,6]);
end


num_exps=ngroups*inner_kfolds*external_kfolds;
sensitivity=zeros(num_exps*length(dim_feat),1);
specificity=sensitivity;
accuracy=sensitivity;
feat_dims=sensitivity;
mean_sen=zeros(1,length(dim_feat));
mean_spe=mean_sen;
mean_acc=mean_sen;
std_sen=mean_sen;
std_spe=mean_sen;
std_acc=mean_sen;

for i=1:length(dim_feat)
    sensitivity(((i-1)*num_exps)+1:i*num_exps)=stat_ord(:,1,i);
    mean_sen(i)=mean(stat_ord(:,1,i));std_sen(i)=std(stat_ord(:,1,i));
    specificity(((i-1)*num_exps)+1:i*num_exps)=stat_ord(:,2,i);
    mean_spe(i)=mean(stat_ord(:,2,i));std_spe(i)=std(stat_ord(:,2,i));
    accuracy(((i-1)*num_exps)+1:i*num_exps)=stat_ord(:,5,i);
    mean_acc(i)=mean(stat_ord(:,5,i));std_acc(i)=std(stat_ord(:,5,i));
    feat_dims(((i-1)*num_exps)+1:i*num_exps)=dim_feat(i);
end

end


function printf_long_results(sel,scores,features_names,num_list,ci)

[~,index]=sort(scores(:,6),'descend');

k_firts=min(num_list,size(scores,1));
fprintf('Order:\tSEN \t\t\t SPE \t\t\t  ACC \t\t\t   ROC \t\t\t    Frecuency\n');
for i=1:k_firts
    fprintf('%d    :  ',i);
    printf_scores_ci(scores(index(i),:),ci(index(i),:));
    fprintf('\n');
end
for i=1:k_firts
    feat=sel(index(i),:)';
    feat=feat(feat>0);
    fprintf('%d: ',i);
    for j=1:length(feat)
        fprintf('%s ',features_names{feat(j)});
    end
    fprintf('\n');
end


end

function printf_scores_ci(scores,ci)
scores=scores([1 2 5:7]);
ci=ci([1:4,9:12]);
for i=1:4
    fprintf('%.1f(%.1f %.1f)  ',scores(i)*100,ci([(i-1)*2+1,i*2])*100);
end
fprintf('%d  ',scores(5));

end

