 function [] = builtPredictiveModels(n_Approach,n_Cov,str_visit,n_typeFeat)
%
% builtPredictiveModels.m
% % n_Approach:   1,2 
% %               1:Two group comparison, 2: Suvival analysis
% % n_Cov:        1,2 
% %               1:Age, 2: Age+gender+education
% % str_visit:    'bl','m06','m12','m18','m24','m36'
% %               
% % n_typeFeat:   1,2 
% %               1:MRI, 2: MRI+NM
% %
% % Building predictive models using survival analysis with age, gender and
% % education as covariantes, by means MRI-based markers and
% % neuropsicological measurements at month 12.
% % builtPredictiveModels(n_Approach,n_Cov,str_visit,n_typeFeat)
%
%
%
% Author: Carlos Platero
% Date: 2019/07/21
% 
% General inquiries: carlos.platero@upm.es
%


typeModels={'LME','extCox_LME'};
varModels=cellstr(typeModels{n_Approach});
typeCovariables={'Ag','AgSxEd'};
varCovar=cellstr(typeCovariables{n_Cov});
typeVisits={'bl','m06','m12','m18','m24','m36'};
typeFeatures={'ROI','ROICAM'};
typeFeat=cellstr(typeFeatures{n_typeFeat});


%% Output
dir_mat=char(strcat('./Results/',varModels,'/'));
file_results=char(strcat('kf_',varModels,varCovar,'_',str_visit,'_',typeFeat));
mat_file=strcat(dir_mat,file_results);

%% Initialization
addpath('./kfolds_aux');
addpath('./data');
addpath(genpath('./external/mRMR'));
addpath(genpath('./external/lme'));
addpath('./external/Survival/univariate');
addpath('./external/Survival');


n_visit=convert_visit_code(typeVisits,str_visit);

%dimension of features willing to check
if(strcmp(typeFeat,'ROI'))
    dim_feat=2:4;
else
    dim_feat=5:7;
end

%number of feature subsets obtained from mRMR for each dimension and each fold:
init_ngr=10;

%number of best combinations used for each dimension and each fold:
ngroups=3;

%inner_kfolds
inner_kfolds=10;

%external_kfolds
external_kfolds=10;

%repeat_kf;
repeat_kf=100;

bsl=true;

%Reserve for 3 best feat groups, all dimensions, 10 external folds.
sel=zeros(ngroups*inner_kfolds*length(dim_feat),max(dim_feat),external_kfolds*repeat_kf);
stat_feat=zeros(ngroups*length(dim_feat)*inner_kfolds,6,2,external_kfolds*repeat_kf);

%% TAKING MATRIX OF DATA
[Tresults,features,feature_names,ni] = getDataADNI(typeFeat);


%% Conversion time
Tresults =getTimeStartStop(Tresults);
times_conv= [Tresults.ConvertTime,Tresults.Tstart,Tresults.Tstop,...
    Tresults.Censoring,Tresults.ID];



%% FIXED EFFECTS MATRIX X
% Defining a fixed effects matrix X for the whole training set, so it
% doesn't interfere with test set.
% Xtrain will be generated as a subset of X

time_visit=Tresults.years;
age=Tresults.Age-time_visit;
group=Tresults.Convert>0;
e4=Tresults.APOE4;
gender=double(strcmp('F',Tresults.sex));
education=Tresults.PTEDUCAT;
viscode=grp2idx(Tresults.VISCODE);
code_visit=unique(viscode);


X=[time_visit,group,age,gender,education,e4,times_conv,viscode];



%% EXTERNAL K-FOLD:
% Inside this loop the train/test division is repeated 10 times. For each
% iteration the function get_LME_models will perform an inner validation
% process
if((strcmp(varModels,'extCox_LME') || strcmp(varModels,'LME')) && n_visit>1)
    time_mask = viscode==code_visit(n_visit);
else
    time_mask = time_visit==0;
end

bsl_mask=time_visit==0;

for j=1:repeat_kf
CVO = cvpartition(length(ni),'KFold',external_kfolds);
for i=1:CVO.NumTestSets
    clc;
    fprintf('Iteration: %d\n',i+(j-1)*external_kfolds);
    
    %% Splitting between train and validation
    [mask_scan_train,mask_scan_test,ni_two_train,ni_two_test]=...
        splitSamples_kfolds(ni,CVO.training(i));
    
    %if all subjects are the same group
    gtrain = group(mask_scan_train & time_mask,:);
    gtest = group(mask_scan_test & time_mask,:);
  
    if(length(unique(gtrain))<2)
        [mask_scan_train,ni_two_train]=addSujectOtherGroup_visit(group,ni,...
            unique(gtrain),mask_scan_train,ni_two_train,bsl_mask,time_mask);
    end
    if(length(unique(gtest))<2)
        [mask_scan_test,ni_two_test]=addSujectOtherGroup_visit(group,ni,...
            unique(gtest),mask_scan_test,ni_two_test,bsl_mask,time_mask);
    end

    
    Train_features=features(mask_scan_train,:);
    Xtrain = X(mask_scan_train,:);
    gtrain = group(mask_scan_train,:);

    
    %% INNER LOOP
    % sel gets the best combination for this training set. The model will
    % be built with the whole training set.
    [sel(:,:,i+(j-1)*external_kfolds)] = Inner_CV_Kfolds_Mul(Xtrain,...
        Train_features,ni_two_train,...
        dim_feat,gtrain,init_ngr,ngroups,inner_kfolds,...
        varModels,varCovar,code_visit(n_visit));
    
    
    % Inside new_groupFeatSelADNI_ROI the model trained with the training
    % set is evaluated with the test set.    
    [stat_feat(:,:,:,i+(j-1)*external_kfolds),~]=...
        train_test_Cox(...
        features,sel(:,:,i),X,...
        mask_scan_train,mask_scan_test,ni_two_train,ni_two_test,bsl,...
        varModels,varCovar,code_visit(n_visit));
    

    if(mod(i+(j-1)*external_kfolds,50)==0)
        save(sprintf('%s_%d.mat',mat_file,i+(j-1)*external_kfolds),'stat_feat','sel','feature_names');
    end
    
end
end

save(sprintf('%s_%d.mat',mat_file,i+(j-1)*external_kfolds),'stat_feat','sel','feature_names');

rmpath('./kfolds_aux');
rmpath('./data');
rmpath(genpath('./external/mRMR'));
rmpath(genpath('./external/lme'));
rmpath('./external/Survival/univariate');
rmpath('./external/Survival');


end


