function demo_predictiveModels
% The predictive models were built using longitudinal data. An extensive 
% set of cortical and subcortical features from MRI data and 
% neuropsychological tests were longitudinally extracted. 
% In this study, the feature selection stage combined filter-based and 
% wrapper-based approaches. Feature subsets of different dimensions were 
% preselected using the mRMR algorithm, and a resampling method searched 
% for each dimension the feature subsets that appears most frequently.
% Subsequently, the proposed feature subsets were evaluated in terms of the 
% cross-validated classification accuracy, and the optimal number of 
% features and specific features to be used in the final models were 
% determined. Two approaches were developed: a) two-group comparison between 
% converter and nonconverter MCI subjects and b) longitudinal survival analysis.  
% Both approaches employed linear mixed effect (LME) modeling to capture the 
% longitudinal trajectories of the markers.
% The two-group comparison approaches used linear discriminant analysis, 
% which were trained with marginal longitudinal trajectory residues. 
% In contrast, the survival analysis used risk ratios obtained
% from the extended Cox model, which were transformed into probabilistic terms 
% of the conversion from MCI to AD using a logistic regression model.
%
% The procedure consisted of two nested CV loops: an inner loop, designed 
% to select the optimal feature set for the proposed models, and an outer 
% loop, designed to obtain an unbiased estimate of model performance.
%
%
% Author: Carlos Platero
% Date: 2019/07/22
% 
% General inquiries: carlos.platero@upm.es
%

%% Build and processing data for predictive models
%
% Two steps: a) Built predicitve models (builtPredictiveModels.m) and
%            b) Processing data (analysis_withSavedData_Mux.m)
%
% builtPredictiveModels.m
% % n_Approach:   1,2 
% %               1:Two group comparison, 2: Suvival analysis
% % n_Cov:        1,2 
% %               1:Age, 2: Age+gender+education
% % str_visit:    'bl','m06','m12','m18','m24','m36'
% %               
% % n_typeFeat:   1,2 
% %               1:MRI, 2: MRI+NM
% %
% % Building predictive models using survival analysis with age, gender and
% % education as covariantes, by means MRI-based markers and
% % neuropsicological measurements at month 12.
% % builtPredictiveModels(n_Approach,n_Cov,str_visit,n_typeFeat)
%
% builtPredictiveModels(2,2,'bl',2);
%
%
% analysis_withSavedData_Mux.m
% % process_data: true,false 
% %               false:Show results, true: processing data
% % n_Approach:   1,2 
% %               1:Two group comparison, 2: Suvival analysis
% % n_Cov:        1,2 
% %               1:Age, 2: Age+gender+education
% % str_visit:    'bl','m06','m12','m18','m24','m36'
% %               
% % n_typeFeat:   1,2 
% %               1:MRI, 2: MRI+NM
% % iter:         iteration of k-fold
% % analysis_withSavedData_Mux(process_data,n_Approach,n_Cov,str_visit,n_typeFeat,iter)
% 
% analysis_withSavedData_Mux(true,2,2,'bl',2,1000);
%

%% Showing results
% % process_data: true,false 
% %               false:Show results, true: processing data
% % n_Approach:   1,2 
% %               1:Two group comparison, 2: Survival analysis
% % n_Cov:        1,2 
% %               1:Age, 2: Age+gender+education
% % str_visit:    'bl','m06','m12','m18','m24','m36'
% %               
% % n_typeFeat:   1,2 
% %               1:MRI, 2: MRI+NM
% % iter:         iteration of k-fold
% % analysis_withSavedData_Mux(process_data,n_Approach,n_Cov,str_visit,n_typeFeat,iter)
%
% analysis_withSavedData_Mux(false,2,2,'bl',2,1000);
% pause;

%% Results 
% % Two group comparison,Age+gender+education, MRI+NM
clc;
fprintf('Results for two group comparison approach with MRI data and\n');
fprintf('neuropsychological measurements (NMs) using age, sex and education\n');
fprintf('as covariates\n');
fprintf('-----------------------------------------------------------------\n');
pause;
analysis_withSavedData_Mux(false,1,2,'bl' ,2,1000);
analysis_withSavedData_Mux(false,1,2,'m12',2,1000);
analysis_withSavedData_Mux(false,1,2,'m24',2,1000);
analysis_withSavedData_Mux(false,1,2,'m36',2,1000);



% % Longitudinal survival analysis,Age+gender+education, MRI+NM
clc;
fprintf('Results for longitudinal survial analysis with MRI data and\n');
fprintf('NMs using age, sex and education');
fprintf('as covariates\n');
fprintf('-----------------------------------------------------------------\n');
pause;
analysis_withSavedData_Mux(false,2,2,'bl', 2,1000);
analysis_withSavedData_Mux(false,2,2,'m12',2,1000);
analysis_withSavedData_Mux(false,2,2,'m24',2,1000);
analysis_withSavedData_Mux(false,2,2,'m36',2,1000);

end