function [] = mRMR_key_features(option,K,mRMRopc)
%% DESCRIPTION
% This function is charged of taking the best features along the cortical
% and subcortical structures based on the mRMR algorithm. 
% options refers to the groups evaluated
% K refers to the number of features selected
% mRMRopc refers to the different implementations of the algorithm:
%       1) difference
%       2) quoeficient
%       3) base

%% DEFAULT VALUES

if nargin <3
    mRMRopc=2;
    if nargin < 2
        K = 5;
        if nargin <1
            option='NC_AD';
        end
    end
end

%% LOAD DATA
data = load('1379data.mat');
addpath(genpath('mi'));

%% COMPUTATION

switch mRMRopc    
    case 1
        analize(data,@mrmr_mid_d,option, K);        
    case 2
        analize(data,@mrmr_miq_d,option, K);        
    case 3        
        analize(data,@mrmr_mibase_d,option, K);        
    otherwise
        disp('please, use a valid option for mRMR implementation.');        
end

end
%% SUPLEMENTARY FUNCTIONS

function analize (data,mRMR_fun,option, K)
%% Numeric asignment for diagnose
% Diagnose of the patient is translated from 'Normal', 'MCI', 'LMCI' or
% 'AD' to 0,1,2,3.
for i = 1: numel(data.Tdec.diagnose)
    y_complete(i) = translate_category (char(data.Tdec.diagnose(i)));
end

%% Subsets of features
X_aseg = data.aseg;
names_aseg =  cellstr(data.asegcols);

X_aparc_lh = data.aparc_lh;
names_aparc_lh = cellstr(data.aparc_lhcols);

X_aparc_rh = data.aparc_rh;
names_aparc_rh = cellstr(data.aparc_rhcols);

%% aseg--------------------------------------------------------------------
[X,y] = select_rows(X_aseg,y_complete,data.Tdec.fsidbase,option);
selection_aseg = mRMR_fun(X, y, K);

%% aparc_lh----------------------------------------------------------------
[X,y] = select_rows(X_aparc_lh,y_complete,data.Tdec.fsidbase,option);
selection_aparc_lh = mRMR_fun(X, y, K);

%% aparc_rh----------------------------------------------------------------
[X,y] = select_rows(X_aparc_rh,y_complete,data.Tdec.fsidbase,option);
selection_aparc_rh = mRMR_fun(X, y, K);

%% display
fprintf('\nSubcortical features:\n');
display_results (selection_aseg, names_aseg);

fprintf('\nCortical features (lh):\n');
display_results (selection_aparc_lh, names_aparc_lh);

fprintf('\nCortical features (rh):\n');
display_results (selection_aparc_rh, names_aparc_rh);

end

function [new_X,new_Y] = select_rows(X,y,fsidbase,option)
%Reduces the size of the matrix in order to take only the 2 groups of
%interest. It also normalizes each feature.

%% Choice of groups
options =strsplit(option, '_');
op1 = translate_category(char(options(1,1)));
op2 = translate_category(char(options(1,2)));

%% Iterating over measures
rows = find(( y == op1 | y == op2 ));
new_X=X(rows,:);
new_Y=y(rows);

%% Alternative using only baseline.
% new_X=[];
% new_Y=[];
% already_used=[];
% for i=1:length(y)
%     if ( y(i)== op1 || y(i) == op2 ) && ~any(strcmp(already_used,fsidbase(i)))
%         new_X = [ new_X ; X(i,:) ];
%         new_Y = [ new_Y ; y(i) ];
%         already_used=[already_used ; fsidbase(i)];
%     end
% end

%% Normalization
new_X = normalize(new_X);
end

function  [ new_diagnose] = translate_category (diagnose)
% conversion into numerical category

if strcmp(diagnose, 'Normal') || strcmp(diagnose, 'NC')
    new_diagnose = 0;
elseif strcmp(diagnose, 'MCI') || strcmp(diagnose, 'LMCI')
    new_diagnose = 1;
elseif strcmp(diagnose, 'AD')
    new_diagnose = 2;
end
end

function [new_X] =normalize(X)
% Normalization of data to be able to work with small values of doubles.

for i=1:size(X,2) 
    new_X(:,i)=  (   X(:,i) - mean( X(:,i) )  ) ./ (2*std( X(:,i )));
end
end

function display_results (selection, names)
% Function charged of printing the selected features.
for i=1:numel(selection)
    fprintf('%ith feature: %s\n',i,names{selection(i)});
end
end