function [final_selection] = Inner_CV_Kfolds_Mul(X,features,ni,dim_feat,...
    group,init_ngr,ngroups,nfolds,varModels,varCovar,varVisit)
% Description: this function is charged of getting the models for different
% dimensions.
%
% Author: Carlos Platero
% Date: 2019/07/21
% 
% General inquiries: carlos.platero@upm.es
%


%% Preparing different dimensions:
size_dim=length(dim_feat);
stat_feat=zeros(init_ngr*length(dim_feat),6,nfolds);
initial_selection=zeros(init_ngr*length(dim_feat),max(dim_feat),nfolds);
reduced_selection = zeros(ngroups*length(dim_feat),max(dim_feat),nfolds);

bsl=true;

time_visit=X(:,1);
bsl_mask = time_visit==0;
viscode=X(:,12);
mask_viscode=viscode==varVisit;

%final selection is just an addaptation of the shape of the selection.
%Instead of using a 3d array, we're joining all of selected groups into a
%matrix in order to evaluate them only once.
%final_selection=zeros(ngroups*nfolds*length(dim_feat),max(dim_feat));
%% Internal loop
CVO = cvpartition(length(ni),'KFold',nfolds);

for i=1:nfolds
    % for each iteration, a new division of the subjects is made, and new
    % 'bests' are possible.
    
    %% Splitting between train and validation
    [mask_scan_train,mask_scan_val,ni_two_train,ni_two_val]=...
        splitSamples_kfolds(ni,CVO.training(i));
    
    gtrain = group(mask_scan_train & bsl_mask,:);
    gtest = group(mask_scan_val & bsl_mask,:);
    
    %if all subjects are the same group
    if(length(unique(gtrain))<2)
        [mask_scan_train,ni_two_train]=addSujectOtherGroup(group,ni,...
            unique(gtrain),mask_scan_train,ni_two_train,bsl_mask);
        
    end
    if(length(unique(gtest))<2)
        [mask_scan_val,ni_two_val]=addSujectOtherGroup(group,ni,...
            unique(gtest),mask_scan_val,ni_two_val,bsl_mask);
    end
    
    %% Choose best features (for each dimension)
    % Is it necessary to do an even deeper loop? K-folds seems to do just
    % once. And mRMR selects the order of the mos suitable features to form
    % a group:
    ni_two_train_mrm=ones(sum(mask_scan_train & bsl_mask),1);
    Train_features=features(mask_scan_train & bsl_mask,:);
    gtrain = group(mask_scan_train & bsl_mask,:);    
    
 

    for n=1:size_dim
        initial_selection( ((n-1)*init_ngr)+1:n*init_ngr ,...
            1:dim_feat(n),i)=featureSelectionADNI_ROI(Train_features,...
            gtrain,ni_two_train_mrm,dim_feat(n),init_ngr);
    end
    
   


    % Inside new_grFeatSelADNI_ROI the train and val subsets are used to
    % perform the training and the validation, respectively. Inner results are
    % not important. They're only used to determine the best group
    % combination.
    gtrain = group(mask_scan_train & mask_viscode,:);
    gtest = group(mask_scan_val & mask_viscode,:);
    %if all subjects are the same group
    if(length(unique(gtrain))<2)
        [mask_scan_train,ni_two_train]=addSujectOtherGroup_visit(group,ni,...
            unique(gtrain),mask_scan_train,ni_two_train,bsl_mask,mask_viscode);
        
    end
    if(length(unique(gtest))<2)
        [mask_scan_val,ni_two_val]=addSujectOtherGroup_visit(group,ni,...
            unique(gtest),mask_scan_val,ni_two_val,bsl_mask,mask_viscode);
    end
    

    [bsl_scores,~]=...
        train_test_Cox(...
        features,initial_selection(:,:,i),X,...
        mask_scan_train,mask_scan_val,ni_two_train,ni_two_val,bsl,...
        varModels,varCovar,varVisit);
    
    stat_feat(:,:,i)=squeeze(bsl_scores(:,:,2));
    reduced_selection(:,:,i) =choosebest_ngroups(stat_feat(:,:,i),...
        initial_selection(:,:,i),ngroups,init_ngr,dim_feat);
end


%% Changing format of selection from reduced to final:

final_selection = format_selection(reduced_selection,ngroups,nfolds,dim_feat);

end



