function [Tresults,features,features_names,ni] = getDataADNI(feat_sel)
    % This function is charged of importing and cleaning the Tresults
    % table, taking only candidate features. However, Tresults is equally
    % returned so cleaned variables are still accessible. Extracted
    % features can be Rois, neuropsycological tests, clusters
    % or all of them.
    %
    %
    % possible values of feat_sel:
    %       'ROI'
    %       'CAM'
    %
    % Author: Carlos Platero
    % Date: 2019/07/21
    % 
    % General inquiries: carlos.platero@upm.es
    %   
    %% Data input
    load('1330','Tresults');
    
    %% BASICS
    % These features are useful in the function.
    time_visit = Tresults.years;
    ni = get_ni(time_visit,numel(unique(Tresults.fsidbase)));
    
    %% MRI ROI markers
    %Combination of both hemispheres (volumes)
    NHV=(Tresults.LHippVol+Tresults.RHippVol)./Tresults.ICV*1e3;
    NAV=(Tresults.LAmygdala+Tresults.RAmigdala)./Tresults.ICV*1e3;
    NPV=(Tresults.LPutamen+Tresults.RPutamen)./Tresults.ICV*1e3;
    NCV=(Tresults.LCaudete+Tresults.RCaudete)./Tresults.ICV*1e3;
    NPaV=(Tresults.LPallidum+Tresults.RPallidum)./Tresults.ICV*1e3;
    
    %Both hemispheres separately (thickness)
    %Left
    LHV=Tresults.LHippVol./Tresults.ICV*1e3;
    LAV=Tresults.LAmygdala./Tresults.ICV*1e3;
    LPV=Tresults.LPutamen./Tresults.ICV*1e3;
    LCV=Tresults.LCaudete./Tresults.ICV*1e3;
    LPaV=Tresults.LPallidum./Tresults.ICV*1e3;
    
    %Right
    RHV=Tresults.RHippVol./Tresults.ICV*1e3;
    RAV=Tresults.RAmigdala./Tresults.ICV*1e3;
    RPV=Tresults.RPutamen./Tresults.ICV*1e3;
    RCV=Tresults.RCaudete./Tresults.ICV*1e3;
    RPaV=Tresults.RPallidum./Tresults.ICV*1e3;
    
    %Combination of both hemispheres (thickness)
    ECT=Tresults.L_ECT+Tresults.R_ECT;
    IPT=Tresults.LinfPar+Tresults.RinfPar;
    ITT=Tresults.LinfTmp+Tresults.RinfTmp;
    MTT=Tresults.LmidTmp+Tresults.RmidTmp;
    PCT=Tresults.LpostCing+Tresults.RpostCing;
    PRT=Tresults.Lprecu+Tresults.Rprecu;
    SPT=Tresults.LsupPar+Tresults.RsupPar;
    TPT=Tresults.LtmpPole+Tresults.RtmpPole;
    MeT=ECT+IPT+ITT+MTT+PCT+PRT+SPT+TPT;
    
    %Both hemispheres separately (thickness)
    %Left
    ECT_LH=Tresults.L_ECT;
    IPT_LH=Tresults.LinfPar;
    ITT_LH=Tresults.LinfTmp;
    MTT_LH=Tresults.LmidTmp;
    PCT_LH=Tresults.LpostCing;
    PRT_LH=Tresults.Lprecu;
    SPT_LH=Tresults.LsupPar;
    TPT_LH=Tresults.LtmpPole;
    
    %Right
    ECT_RH=Tresults.R_ECT;
    IPT_RH=Tresults.RinfPar;
    ITT_RH=Tresults.RinfTmp;
    MTT_RH=Tresults.RmidTmp;
    PCT_RH=Tresults.RpostCing;
    PRT_RH=Tresults.Rprecu;
    SPT_RH=Tresults.RsupPar;
    TPT_RH=Tresults.RtmpPole;
    
    feat_Roi=[NHV, NAV, NPV, NCV, NPaV,...
        LHV, LAV, LPV, LCV, LPaV,...
        RHV, RAV, RPV, RCV,RPaV,...
        ECT, IPT, ITT, MTT,PCT,...
        PRT, SPT, TPT, MeT,ECT_LH,...
        IPT_LH, ITT_LH, MTT_LH, PCT_LH, PRT_LH,...
        SPT_LH, TPT_LH, ECT_RH, IPT_RH, ITT_RH,...
        MTT_RH, PCT_RH, PRT_RH, SPT_RH, TPT_RH];
    
    feat_Roi_names={'NHV','NAV','NPV','NCV','NPaV',...
        'LHV','LAV','LPV','LCV','LPaV',...
        'RHV','RAV','RPV','RCV','RPaV',...
        'ECT','IPT','ITT','MTT','PCT',...
        'PRT','SPT','TPT','MeT','ECT_LH',...
        'IPT_LH','ITT_LH','MTT_LH','PCT_LH','PRT_LH',...
        'SPT_LH','TPT_LH','ECT_RH','IPT_RH','ITT_RH',...
        'MTT_RH','PCT_RH','PRT_RH','SPT_RH','TPT_RH'};
    
    
    %clear variables for debugging
    clear NHV NAV NPV NCV NPaV ...
        LHV LAV LPV LCV LPaV ...
        RHV RAV RPV RCV RPaV ...
        ECT IPT ITT MTT PCT ...
        PRT SPT TPT MeT ECT_LH ...
        IPT_LH ITT_LH MTT_LH PCT_LH PRT_LH ...
        SPT_LH TPT_LH ECT_RH IPT_RH ITT_RH ...
        MTT_RH PCT_RH PRT_RH SPT_RH TPT_RH;
    
    
    %% CAM MARKERS
    feat_CAM = [Tresults.RAVLT_forgetting,...%cell2mat(Tresults.RAVLT_forgetting),...
        Tresults.RAVLT_immediate,...
        Tresults.RAVLT_learning,...
        Tresults.RAVLT_perc_forgetting,...%? k paso?
        Tresults.ADAS11,...
        Tresults.ADAS13,...
        Tresults.FAQ,...
        Tresults.MMSE,...
        Tresults.GDS,...
        Tresults.CDR,...
        Tresults.ADASQ4];

    
    feat_CAM_names = {'RAVLT_forgetting',...
        'RAVLT_immediate',...
        'RAVLT_learning',...
        'RAVLT_perc_forgetting',...
        'ADAS11',...
        'ADAS13',...
        'FAQ',...
        'MMSE',...
        'GDS',...
        'CDR',...
        'ADASQ4'};

    % % Se deberia interpolar algunos valores de RAVLT_perc_forgetting y ADAS13
    feat_CAM = clean_invalid_CAM(feat_CAM,ni,time_visit);
    
    
    
    
    %% Appropriate combination
    
    if strcmp(feat_sel,'ROI')
        features = feat_Roi;
        features_names = feat_Roi_names;
    elseif strcmp(feat_sel,'ROICAM')
        features = [feat_Roi, feat_CAM];
        features_names = [feat_Roi_names, feat_CAM_names];
        
    else
        error('Invalid input statement. feat_sel must be ROI, CAM, MASS or all');
    end
    
    
end




