function [stat_feat,gr_feat]=...
    train_test_Cox(...
    features,selectedFeat,X,...
    mask_scan_train,mask_scan_test,ni_two_train,ni_two_test,bsl,...
    varModels,varCovar,varVisit)
% This function is charged of performing LME calculating the longitudinal
% residues with the training set and evaluating the model with the test
% set. It is performed just once (no nboot) for each call.
%
% Author: Carlos Platero
% Date: 2019/07/21
% 
% General inquiries: carlos.platero@upm.es
%
%% group of features
gr_feat = selectedFeat;
selectedFeat = unique(selectedFeat)';
if(selectedFeat(1)==0)
    selectedFeat=selectedFeat(2:end);
end
gr_feat=convertFeatures(gr_feat,selectedFeat);

features=features(:,selectedFeat);

%% Covariates
%X=[time_visit,group,age,gender,education,e4,times_conv];
if(strcmp(varCovar,'Ag'))
    covariables=X(:,3);
elseif(strcmp(varCovar,'AgSxEd'))
    covariables=X(:,3:5); %Age+sex+edu
end

%% Cox: Baseline mask
numSamples=size(features,1);
time_visit=X(:,1);
group_diagn=X(:,2);
bsl_mask = time_visit==0;
%mask_test_bsl = mask_scan_test  & bsl_mask;

    
if(strcmp(varModels,'LME'))
    mask_train_bsl= mask_scan_train & bsl_mask;
    [mean_features,std_features]=balanceMeanStd(features(mask_train_bsl,:),group_diagn(mask_train_bsl));
    features=(features-repmat(mean_features,numSamples,1))./repmat(std_features,numSamples,1);

    if(strcmp(varCovar,'Ag'))
        X_design=[ones(numSamples,1),X(:,1),X(:,2),X(:,1).*X(:,2),X(:,3),X(:,end)];
    elseif(strcmp(varCovar,'AgSxEd'))
        X_design=[ones(numSamples,1),X(:,1),X(:,2),X(:,1).*X(:,2),X(:,3:5),X(:,end)];
    end
    [error_train,error_test,diagn_two_GT_train,diagn_two_GT_test]=...
        getLMEmulti_visits(X_design,features,mask_scan_train,mask_scan_test,...
        ni_two_train,ni_two_test,bsl,varVisit);
    
elseif(strcmp(varModels,'extCox_LME'))
     Convert=X(:,2); %pMCI=1, sMCI=0
     ConverTime=X(:,7);
     ID=X(:,11);
     viscode=X(:,12);
     
     mask_control=mask_scan_train & (viscode==varVisit);
     [mean_features,std_features]=balanceMeanStd(features(mask_control,:),group_diagn(mask_control));
     features=(features-repmat(mean_features,numSamples,1))./repmat(std_features,numSamples,1);
     [mean_covariates,std_covariates]=balanceMeanStd(covariables(mask_control,:),group_diagn(mask_control));
     covariables=(covariables-repmat(mean_covariates,numSamples,1))./repmat(std_covariates,numSamples,1);


     if(strcmp(varCovar,'Ag'))
        X_design=[ones(numSamples,1),X(:,1),X(:,2),X(:,1).*X(:,2),X(:,3)];
     elseif(strcmp(varCovar,'AgSxEd'))
        X_design=[ones(numSamples,1),X(:,1),X(:,2),X(:,1).*X(:,2),X(:,3:5)];
     end
     
     [Markers_estimated,d_ext,t_ext,~,~,~]=getExtCoxLMEmulti_visits(X_design,...
         features,mask_scan_train,ni_two_train,Convert,ConverTime,ID,...
         covariables);
     
else   
end



%% Classification
num_groups=size(gr_feat,1);
stat_feat=zeros(num_groups,6,2);

for j=1:num_groups
    featSorted=unique(gr_feat(j,:));
    if(featSorted(1)==0)
        featSorted=featSorted(2:end);
    end
    
    if(strcmp(varModels,'LME'))
       stat_feat(j,:,:)=getClassifierLMEmulti(error_train(:,featSorted),...
            error_test(:,featSorted,:),diagn_two_GT_train,diagn_two_GT_test,bsl);
        
    elseif(strcmp(varModels,'extCox_LME'))
         
        features_test= features(mask_scan_test &  (viscode==varVisit), featSorted);
        group_two_test_visit=X(mask_scan_test &  (viscode==varVisit),2);
        
        try
            
            [~,~,~,cox_model] = coxphfit(Markers_estimated(:,featSorted),t_ext,'censoring',d_ext==0);
            scores=classifierLogistCox(cox_model.beta,features_test,group_two_test_visit);       
           

            stat_feat(j,:,2)=scores;        
            stat_feat(j,:,1)=scores;
       catch
           scores=zeros(1,6);
           stat_feat(j,:,1)=scores;
           stat_feat(j,:,2)=scores;
       end
        
    end
    

   
end



end

function acc=classifierLogistCox(beta,features,group_two_test)
p_Conv_bsl=1./(1+exp(-features*beta));
acc=qualityClassifier(group_two_test>0,p_Conv_bsl);

end


function acc=qualityClassifier(diagn_two_GT,pr_pMCI)
% Quality
[C_all,~] = confusionmat(diagn_two_GT,pr_pMCI>.5);
if(length(C_all(:))==4)
    TN=C_all(1,1);
    FN=C_all(2,1);
    FP=C_all(1,2);
    TP=C_all(2,2);
    sensitivy=TP/(TP+FN);
    specificity=TN/(TN+FP);
    PPV = TP/(TP+FP);
    NPV= TN/(TN+FN);
    ACC= (TN+TP)/(TN+TP+FN+FP);
    if(length(unique(diagn_two_GT))==2)
        [~,~,~,auc]=perfcurve(diagn_two_GT,pr_pMCI,1);
    else
        auc=ACC;
    end
    acc=[sensitivy,specificity,PPV,NPV,ACC,auc];
else
    acc=zeros(1,6);
end
end


function [mean_features,std_features]=balanceMeanStd(features,group_diagn)

labels=unique(group_diagn);
num_label_1=sum(group_diagn==labels(1));
num_label_2=sum(group_diagn==labels(2));
mask_balance=true(length(group_diagn),1);

if(num_label_1>num_label_2)
    num_delete=num_label_1-num_label_2;
    index=find(group_diagn==labels(1));
    ran=randperm(num_label_1);
    mask_balance(index(ran(1:num_delete)))=false;       
elseif(num_label_2>num_label_1)
    num_delete=num_label_2-num_label_1;
    index=find(group_diagn==labels(2));
    ran=randperm(num_label_2);
    mask_balance(index(ran(1:num_delete)))=false;
else
end


mean_features=mean(features(mask_balance,:));
std_features=std(features(mask_balance,:));

end

